@@ -3,8 +3,14 @@ import warnings
 from typing import Any
 
 
+_FILTERS_INSTALLED = False
+
+
 def add_warning_filters() -> None:
-    # TODO: This should be removed when we have fixed the error messages in the LiteLLM library
+    global _FILTERS_INSTALLED
+    if _FILTERS_INSTALLED:
+        return
+
     warnings.filterwarnings("ignore", message="coroutine '.*' was never awaited")
     warnings.filterwarnings(
         "ignore",
@@ -19,9 +25,15 @@ def add_warning_filters() -> None:
             )
 
     # Add filter to multiple possible loggers
-    filter = NoEventLoopFilterLiteLLM()
-    logging.getLogger("LiteLLM").addFilter(filter)
+    lite_logger = logging.getLogger("LiteLLM")
+    has_filter = any(
+        isinstance(f, NoEventLoopFilterLiteLLM) for f in lite_logger.filters
+    )
+    if not has_filter:
+        lite_logger.addFilter(NoEventLoopFilterLiteLLM())
 
     import litellm
 
-    litellm.suppress_debug_info = True  # to disable colorized prints with links to litellm whenever an LLM provider raises an error
+    litellm.suppress_debug_info = True
+
+    _FILTERS_INSTALLED = True
