@@ -6,7 +6,8 @@ import opik.dict_utils as dict_utils
 from opik.api_objects import span
 from opik.decorator import arguments_helpers, base_track_decorator
 
-from . import helpers, stream_wrappers
+from . import types
+from .converse import stream_wrappers as converse_stream_wrappers
 
 LOGGER = logging.getLogger(__name__)
 
@@ -77,7 +78,7 @@ class BedrockInvokeAgentDecorator(base_track_decorator.BaseTrackDecorator):
         capture_output: bool,
         generations_aggregator: Optional[Callable[[List[Any]], Any]],
     ) -> Union[
-        helpers.ConverseStreamOutput,
+        types.ConverseStreamOutput,
         None,
     ]:
         DECORATED_FUNCTION_IS_NOT_EXPECTED_TO_RETURN_GENERATOR = (
@@ -92,7 +93,7 @@ class BedrockInvokeAgentDecorator(base_track_decorator.BaseTrackDecorator):
         if isinstance(output, dict) and "completion" in output:
             span_to_end, trace_to_end = base_track_decorator.pop_end_candidates()
 
-            wrapped_stream = stream_wrappers.wrap_stream(
+            wrapped_stream = converse_stream_wrappers.wrap_stream(
                 stream=output["completion"],
                 capture_output=capture_output,
                 span_to_end=span_to_end,
@@ -103,7 +104,7 @@ class BedrockInvokeAgentDecorator(base_track_decorator.BaseTrackDecorator):
             )
 
             output["completion"] = wrapped_stream
-            return cast(helpers.ConverseStreamOutput, output)
+            return cast(types.ConverseStreamOutput, output)
 
         STREAM_NOT_FOUND = None
 
