@@ -1,7 +1,7 @@
 import dataclasses
 import datetime
 from dataclasses import field
-from typing import Optional, Any, Dict, List, Union, Literal
+from typing import Optional, Any, Dict, List, Union, Literal, Set
 
 from . import arguments_utils
 from ..rest_api.types import span_write, trace_write
@@ -50,6 +50,10 @@ class CreateTraceMessage(BaseMessage):
         data["id"] = data.pop("trace_id")
         return data
 
+    @staticmethod
+    def fields_to_anonymize() -> Set[str]:
+        return {"input", "output", "metadata"}
+
 
 @dataclasses.dataclass
 class UpdateTraceMessage(BaseMessage):
@@ -78,6 +82,10 @@ class UpdateTraceMessage(BaseMessage):
         data["id"] = data.pop("trace_id")
         return data
 
+    @staticmethod
+    def fields_to_anonymize() -> Set[str]:
+        return {"input", "output", "metadata"}
+
 
 @dataclasses.dataclass
 class CreateSpanMessage(BaseMessage):
@@ -112,6 +120,10 @@ class CreateSpanMessage(BaseMessage):
         data["total_estimated_cost"] = data.pop("total_cost")
         return data
 
+    @staticmethod
+    def fields_to_anonymize() -> Set[str]:
+        return {"input", "output", "metadata"}
+
 
 @dataclasses.dataclass
 class UpdateSpanMessage(BaseMessage):
@@ -144,6 +156,10 @@ class UpdateSpanMessage(BaseMessage):
         data["total_estimated_cost"] = data.pop("total_cost")
         return data
 
+    @staticmethod
+    def fields_to_anonymize() -> Set[str]:
+        return {"input", "output", "metadata"}
+
 
 @dataclasses.dataclass
 class FeedbackScoreMessage(BaseMessage):
@@ -200,11 +216,19 @@ class AddThreadsFeedbackScoresBatchMessage(BaseMessage):
 class CreateSpansBatchMessage(BaseMessage):
     batch: List[span_write.SpanWrite]
 
+    @staticmethod
+    def fields_to_anonymize() -> Set[str]:
+        return {"input", "output", "metadata"}
+
 
 @dataclasses.dataclass
 class CreateTraceBatchMessage(BaseMessage):
     batch: List[trace_write.TraceWrite]
 
+    @staticmethod
+    def fields_to_anonymize() -> Set[str]:
+        return {"input", "output", "metadata"}
+
 
 @dataclasses.dataclass
 class GuardrailBatchItemMessage(BaseMessage):
